@@ -63,9 +63,9 @@ A trick to invalidate a single value:
 """
 
 
+from decorator import decorate
 import collections as _collections
 import datetime as _datetime
-import functools as _functools
 import inspect as _inspect
 import os as _os
 import cPickle as _pickle
@@ -155,7 +155,8 @@ def _args_key(function, args, kwargs, function_key=function_name):
             #       because shelve only works with proper strings.
             #       Otherwise, we'd get an exception because
             #       function.__name__ is str but dumps returns bytes.
-            arguments_pickle = _pickle.dumps(arguments, protocol=0).decode('ascii')
+            arguments_pickle = _pickle.dumps(
+                arguments, protocol=0).decode('ascii')
     except TypeError as e:
         raise PicklingError(str(e))
 
@@ -225,7 +226,7 @@ class FileBackend(object):
                 _pickle.dump(value, file, _pickle.HIGHEST_PROTOCOL)
             except TypeError as e:
                 raise PicklingError(str(e))
-        except portalocker.LockException, e:
+        except portalocker.LockException as e:
             # someone else had the lock, thats ok, we don't have to
             # write the value if someone else already is
             pass
@@ -320,7 +321,54 @@ class SkipCache(Exception):
         self.retval = retval
 
 
-def functioncache(seconds_of_validity=None, fail_silently=True, backend=ShelveBackend(), ignore_instance=False, function_key=function_name):
+def function_with_cache(function, *args, **kwargs):
+    try:
+        key = _args_key(
+            function, args[1:] if function._ignore_instance else args, kwargs,
+            function_key=function._function_key
+        )
+
+        if key in function._db:
+            rv = function._db[key]
+            if function._seconds_of_validity is None or _time.time(
+            ) - rv.timesig < function._seconds_of_validity:
+                return rv.data
+    except:
+        # in any case of failure, don't let functioncache break the
+        # program
+        error_str = _traceback.format_exc()
+        _log_error(error_str)
+        if not function._fail_silently:
+            raise
+
+    try:
+        retval = function(*args, **kwargs)
+    # Log the error, return the value, don't cache it.
+    except SkipCache as e:
+        error_str = _traceback.format_exc()
+        _log_error(error_str)
+        return e.retval
+
+    # store in cache
+    try:
+        function._db[key] = _retval(_time.time(), retval)
+    except:
+        # in any case of failure, don't let functioncache break the
+        # program
+        error_str = _traceback.format_exc()
+        _log_error(error_str)
+        if not function._fail_silently:
+            raise
+
+    return retval
+
+
+def functioncache(
+        seconds_of_validity=None,
+        fail_silently=True,
+        backend=ShelveBackend(),
+        ignore_instance=False,
+        function_key=function_name):
     '''
     functioncache is called and the decorator should be returned.
 
@@ -331,47 +379,6 @@ def functioncache(seconds_of_validity=None, fail_silently=True, backend=ShelveBa
         backend = backend()
 
     def functioncache_decorator(function):
-        @_functools.wraps(function)
-        def function_with_cache(*args, **kwargs):
-            try:
-                key = _args_key(
-                    function, args[1:] if ignore_instance else args, kwargs,
-                    function_key=function_key
-                )
-
-                if key in function._db:
-                    rv = function._db[key]
-                    if seconds_of_validity is None or _time.time() - rv.timesig < seconds_of_validity:
-                        return rv.data
-            except:
-                # in any case of failure, don't let functioncache break the
-                # program
-                error_str = _traceback.format_exc()
-                _log_error(error_str)
-                if not fail_silently:
-                    raise
-
-            try:
-                retval = function(*args, **kwargs)
-            # Log the error, return the value, don't cache it.
-            except SkipCache, e:
-                error_str = _traceback.format_exc()
-                _log_error(error_str)
-                return e.retval
-
-            # store in cache
-            try:
-                function._db[key] = _retval(_time.time(), retval)
-            except:
-                # in any case of failure, don't let functioncache break the
-                # program
-                error_str = _traceback.format_exc()
-                _log_error(error_str)
-                if not fail_silently:
-                    raise
-
-            return retval
-
         # make sure cache is loaded
         if not hasattr(function, '_db'):
             cache_name = (_get_cache_name(function), type(backend))
@@ -384,9 +391,14 @@ def functioncache(seconds_of_validity=None, fail_silently=True, backend=ShelveBa
 
             function_with_cache._db = function._db
 
-        return function_with_cache
+        function._seconds_of_validity = seconds_of_validity
+        function._fail_silently = fail_silently
+        function._ignore_instance = ignore_instance
+        function._function_key = function_key
 
-    if type(seconds_of_validity) == _types.FunctionType:
+        return decorate(function, function_with_cache)
+
+    if isinstance(seconds_of_validity, _types.FunctionType):
         # support for when people use '@functioncache.functioncache' instead of
         # '@functioncache.functioncache()'
         func = seconds_of_validity
